#include <gtk/gtk.h>
#include "playlist.h"

enum {
    TITLE_COLUMN,
    ARTIST_COLUMN,
    FILE_COLUMN,
    N_COLUMNS
};

static GtkWindowClass *parent_class = NULL;

static void playlist_init (PlayList *list);
static void playlist_class_init (PlayListClass *klass);
static void playlist_finalize (GObject *object);

static gboolean playlist_delete_event_cb (GtkWidget *widget, GdkEvent *event, PlayList *list);
static gboolean playlist_add_cb(GtkButton *button, PlayList *list);
static gboolean playlist_stop_cb(GtkButton *button, PlayList *list);
static gboolean playlist_play_cb(GtkButton *button, PlayList *list);

GType
playlist_get_type (void)
{
    static GType our_type = 0;
    
    if (our_type == 0) {
	static const GTypeInfo our_info = {
	    sizeof (PlayListClass),
	    NULL,
	    NULL,
	    (GClassInitFunc) playlist_class_init,
	    NULL,
	    NULL,
	    sizeof (PlayList),
	    0,
	    (GInstanceInitFunc) playlist_init
	};
	our_type = g_type_register_static (GTK_TYPE_WINDOW, "PlayList", &our_info, 0);
    }
    
    return our_type;
}

static void
playlist_init (PlayList *list)
{
    GtkWidget *b, *hbox;
    GtkCellRenderer *renderer;
    GtkTreeViewColumn *column;

    g_signal_connect(G_OBJECT(list), "delete_event",
		     G_CALLBACK(playlist_delete_event_cb), list);

    list->vbox = gtk_vbox_new (FALSE, 0);
    gtk_container_add (GTK_CONTAINER (list), list->vbox);

    list->view = gtk_tree_view_new();
    gtk_box_pack_start (GTK_BOX(list->vbox), list->view, FALSE, FALSE, 0);

    hbox = gtk_hbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX(list->vbox), hbox, FALSE, FALSE, 0);

    b = gtk_button_new_with_label ("Play");
    g_signal_connect(G_OBJECT(b), "clicked",
		     G_CALLBACK(playlist_play_cb), list);
    gtk_box_pack_start (GTK_BOX(hbox), b, FALSE, FALSE, 0);

    b = gtk_button_new_with_label ("Stop");
    g_signal_connect(G_OBJECT(b), "clicked",
		     G_CALLBACK(playlist_stop_cb), list);
    gtk_box_pack_start (GTK_BOX(hbox), b, FALSE, FALSE, 0);

    b = gtk_button_new_with_label ("Add");
    g_signal_connect(G_OBJECT(b), "clicked",
		     G_CALLBACK(playlist_add_cb), list);
    gtk_box_pack_start (GTK_BOX(hbox), b, FALSE, FALSE, 0);

    list->store = gtk_list_store_new (N_COLUMNS,
				      G_TYPE_STRING,
				      G_TYPE_STRING,
				      G_TYPE_STRING);
    gtk_tree_view_set_model (GTK_TREE_VIEW(list->view), GTK_TREE_MODEL(list->store));

    renderer = gtk_cell_renderer_text_new ();
    column = gtk_tree_view_column_new_with_attributes ("Title",
						       renderer,
						       "text", TITLE_COLUMN,
						       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (list->view), column);

    column = gtk_tree_view_column_new_with_attributes ("Artist",
						       renderer,
						       "text", ARTIST_COLUMN,
						       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (list->view), column);
}

static void
playlist_class_init (PlayListClass *klass)
{
    GObjectClass *gobject_class = (GObjectClass *)klass;
//    GtkWidgetClass *widget_class = (GtkWidgetClass *)klass;

    parent_class = g_type_class_peek_parent (klass);

    gobject_class->finalize = playlist_finalize;

//    widget_class->delete_event = playlist_delete;
}

static void
playlist_finalize (GObject *object)
{
}

GtkWidget *
playlist_new ()
{
    PlayList *list;

    list = g_object_new (PLAYLIST_TYPE, NULL);

    return GTK_WIDGET (list);
}

static gboolean
playlist_delete_event_cb (GtkWidget *widget, GdkEvent *event, PlayList *list)
{
    gtk_widget_hide_all (GTK_WIDGET(list));
    return TRUE;
}

static void
store_filename(GtkWidget *widget, GtkWidget *fs)
{
    const gchar *filename;
    PlayList *list;
    GtkTreeIter iter;

    filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs));
    g_print ("%s\n", filename);

    list = (PlayList *) g_object_get_data (G_OBJECT(fs), "list");
    
    // TODO: get title, artist from file. use id3tag(mp3), libvorbis(ogg)
    gtk_list_store_append(list->store, &iter);
    gtk_list_store_set (list->store, &iter,
			TITLE_COLUMN, filename,
			ARTIST_COLUMN, "",
			FILE_COLUMN, filename,
			-1);
}

static gboolean
playlist_add_cb (GtkButton *button, PlayList *list)
{
    GtkWidget *fs;

    fs = gtk_file_selection_new ("select files");

    g_object_set_data (G_OBJECT(fs), "list", (gpointer)list);

    g_signal_connect (G_OBJECT(GTK_FILE_SELECTION(fs)->ok_button), "clicked",
		     G_CALLBACK(store_filename), fs);
    g_signal_connect_swapped (G_OBJECT (GTK_FILE_SELECTION (fs)->ok_button), "clicked",
                             G_CALLBACK (gtk_widget_destroy), fs); 
    g_signal_connect_swapped (G_OBJECT (GTK_FILE_SELECTION (fs)->cancel_button), "clicked",
                             G_CALLBACK (gtk_widget_destroy), fs); 

    gtk_widget_show (fs);
}

static gboolean
playlist_stop_cb (GtkButton *button, PlayList *list)
{
    
}

static gboolean
playlist_play_cb (GtkButton *button, PlayList *list)
{
    
}
