#include <gtk/gtk.h>
#include "tray.h"
#include "eggtrayicon.h"

enum {
    QUIT,
    BUTTON_PRESS,
    N_SIGNALS
};

static guint tray_signals[N_SIGNALS] = { 0 };

static EggTrayIconClass *parent_class = NULL;

static void tray_init (Tray *tray);
static void tray_class_init (TrayClass *klass);
static void tray_finalize (GObject *object);

static void tray_press_cb (GtkWidget *widget, GdkEventButton *event, Tray *tray);

static void quit_cb(GtkMenuItem *item, Tray *tray);

GType
tray_get_type (void)
{
    static GType our_type = 0;
    
    if (our_type == 0) {
	static const GTypeInfo our_info = {
	    sizeof (TrayClass),
	    NULL,
	    NULL,
	    (GClassInitFunc) tray_class_init,
	    NULL,
	    NULL,
	    sizeof (Tray),
	    0,
	    (GInstanceInitFunc) tray_init
	};
	our_type = g_type_register_static (EGG_TYPE_TRAY_ICON, "Tray", &our_info, 0);
    }
    
    return our_type;
}

static void
tray_init (Tray *tray)
{
    GtkWidget *t;

    tray->eb = gtk_event_box_new ();
    g_signal_connect (G_OBJECT(tray->eb),
		      "button-press-event",
		      G_CALLBACK(tray_press_cb),
		      tray);
    gtk_container_add (GTK_CONTAINER(tray), tray->eb);

    tray->label = gtk_label_new("Amp");
    gtk_container_add(GTK_CONTAINER(tray->eb), tray->label);

    /* popup menu */
    tray->menu = gtk_menu_new();
    t = gtk_menu_item_new_with_label("quit");
    g_signal_connect(G_OBJECT(t), "activate",
		     G_CALLBACK(quit_cb), tray);
    gtk_menu_append(tray->menu, t);
    gtk_widget_show_all(tray->menu);
}

static void
tray_class_init (TrayClass *klass)
{
    GObjectClass *gobject_class = (GObjectClass *)klass;
    parent_class = g_type_class_peek_parent (klass);
    gobject_class->finalize = tray_finalize;

    klass->quit = NULL;
    klass->button_press = NULL;

    tray_signals[QUIT] = 
	g_signal_new ("quit", G_TYPE_FROM_CLASS(klass),
		      G_SIGNAL_RUN_FIRST,
		      G_STRUCT_OFFSET (TrayClass, quit),
		      NULL, NULL,
		      g_cclosure_marshal_VOID__VOID,
		      G_TYPE_NONE,
		      0);
    tray_signals[BUTTON_PRESS] = 
	g_signal_new ("button_press", G_TYPE_FROM_CLASS(klass),
		      G_SIGNAL_RUN_FIRST,
		      G_STRUCT_OFFSET (TrayClass, button_press),
		      NULL, NULL,
		      g_cclosure_marshal_VOID__VOID,
		      G_TYPE_NONE,
		      0);
}

static void
tray_finalize (GObject *object)
{
}

GtkWidget *
tray_new ()
{
    Tray *tray;

    tray = g_object_new (TRAY_TYPE, NULL);

    return GTK_WIDGET (tray);
}

static void
tray_press_cb (GtkWidget *widget,
	       GdkEventButton *event,
	       Tray *tray)
{
    if (event->type == GDK_BUTTON_PRESS && event->button == 3) {
	gtk_menu_popup(GTK_MENU(tray->menu), NULL, NULL, NULL, NULL,
		       event->button, event->time);
    } else if (event->type == GDK_2BUTTON_PRESS && event->button == 1) {
	g_signal_emit(tray, tray_signals[BUTTON_PRESS], 0);
    }
}

static void
quit_cb(GtkMenuItem *item, Tray *tray)
{
    g_signal_emit(tray, tray_signals[QUIT], 0);
}
